@@ -10,6 +10,7 @@
 #' @family library functions
 
 lib_status <- function(lib = .libPaths()[1]) {
+  load_extra("tibble")
   remote(
     function(...) asNamespace("pak")$lib_status_internal(...),
     list(lib = lib))
@@ -29,5 +30,5 @@ lib_status <- function(lib = .libPaths()[1]) {
 ## ----------------------------------------------------------------------
 
 lib_status_internal <- function(lib) {
-  pkgdepends_lib_status(lib)
+  pkgdepends::lib_status(lib)
 }
